:: File: RestoreTMSM.bat
:: Version: 1.0
:: Purpose: Retain the TMSM Plug-in settings after migration or reinstallation.
::
:: Please specify the prameters
:: -b : Backup TMSM files
:: -rc: Restore TMSM files
:: -ra: Backup and restore TMSM files (Supported after TMSM 2.0)
::
:: ******** Note ********
:: 1.Please use PSTool (PsExec.exe -s -i cmd.exe) to open the cmd with system privilege then run this batch file.
:: 2.Please ensure you have login Windows as administrator session.

:: ******** Usage ********
:: There are two ways to use the tool:
:: 1.Before uninstall TMSM, backup certificate files via argument '-b'
::   After install TMSM, restore certificate files via argument '-rc'
:: 2.After TMSM 2.0, system automatically backed up certificate files when uninstall or upgrade TMSM, we can use argument '-ra' to backup and restore at one time.
:: 	Note :	Command '-ra' will not backed up TMSM database.
:: 			
:: TMSM certificate files will be copied(folder name is timestamp) before overwrite backed up one.
::
:: We suggest user backup certificated files via argument '-b' to copy related files automatically.
:: If you need to backup certificated files manually, please store the certificate files as the format below and place in the same folder with this batch file.
::	TMSM\
::		ServerInfo.plist
::		apache-activemq\
::						bin\
::							win32\ OR win64\
::											wrapper.conf
::						conf\
::							activemq.xml
::							broker.cert
::							broker.pem
::							broker.ks
::							
:: 


@echo off
:: ******** Init parameters *********
if "%PROCESSOR_ARCHITECTURE%"=="AMD64" (
	set WOW6432NODE=Wow6432Node\
) else (
	set WOW6432NODE=
)

:: Get OSCE path from registry key
reg query "HKEY_LOCAL_MACHINE\SOFTWARE\%WOW6432NODE%TrendMicro\OfficeScan\service\Information" /v "Local_Path" >nul 2>&1
if %errorlevel% NEQ 0 (
	echo Can't find OSCE local path.
	goto:END
)
for /F "tokens=2*" %%A in ('reg query "HKEY_LOCAL_MACHINE\SOFTWARE\%WOW6432NODE%TrendMicro\OfficeScan\service\Information" /v "Local_Path"') do (set PCCSRVPATH=%%B)
set OSCE_PATH=%PCCSRVPATH%..
set TMSM_PATH=%OSCE_PATH%\Addon\TMSM
set TMSM_BACKUP_PATH=%PCCSRVPATH%Private\TMSM_CERT_BACKUP
set ACTIVEMQ_PATH=activemq.xml
set BROKER_PEM_PATH=broker.pem
set BROKER_CERT_PATH=broker.cert
set BROKER_KS_PATH=broker.ks
set WRAPPER_CONF_PATH=wrapper.conf
set WRAPPER32_CONF_PATH=wrapper_win32.conf
set WRAPPER64_CONF_PATH=wrapper_win64.conf
set TEMP_WRAPPER32_CONF_PATH=wrapper_win32.conf.temp
set TEMP_WRAPPER64_CONF_PATH=wrapper_win64.conf.temp
set SERVERINFO_PLIST_PATH=ServerInfo.plist
set APACHE_CONF_PATH=apache-activemq\conf
set APACHE_BIN_PATH=apache-activemq\bin
set APACHE_BIN_WIN32=apache-activemq\bin\win32
set APACHE_BIN_WIN64=apache-activemq\bin\win64
set APACHE_BIN_FILE=activemq.jar
set BACKUP_TMSM_FOLDER=TMSM
set TMSMINSTALL_MPKG_PATH=%TMSM_PATH%\TMSM_HTML\ActiveUpdate\ClientInstall\tmsminstall.mpkg.zip
set TMSMINSTALL_PATH=%TMSM_PATH%\TMSM_HTML\ActiveUpdate\ClientInstall\tmsminstall.zip
set INATALL_MPKG_ZIP_PATH=tmsminstall.mpkg.zip
set TEMP_ACTIVEMQ_PATH=activemq.xml.temp
set TEMP_OSNAME_PATH=_OSName.temp
set TMSMSQL_BAK=TMSMSQL.bak
set TMSMSQL_SERVICES=MSSQL$TMSM
set COPY_VBS=_Copy.vbs
set SUBSTRING_VBS=_SubString.vbs
set COMPAREFILESSUCCESS=true
set ISTMSM_15=false
set ISTMSM_30=false
set IsBackupDB=false
set IsRestoreDB=false
set IsDeleteBackup=false
set OS2k8OrAfter=true
::Arguments call by ServerInfo tool
set SERVERINFO_PATH=%TMSM_PATH%\ServerInfoTool.exe
set PROTOCOL=ssl
set OSCEINI_PATH=%PCCSRVPATH%ofcscan.ini
set MQ_CONFIG_PATH=%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%
set CLIENTINSTALL_PATH=%TMSM_PATH%\TMSM_HTML\ActiveUpdate\ClientInstall
set PRODUCT_PATH=%TMSM_PATH%\TMSM_HTML\ActiveUpdate\product
set SERVERINI_PATH=%TMSM_PATH%\TMSM_HTML\ActiveUpdate\server.ini
set TMSM_HOME_DIR_PATH=%TMSM_PATH%
set ISUPGRADEDFROM_15=false
:: ******** Get java version ********
set ISINSTALLED_JRE64=
java -d64 -version >nul 2>&1
if errorlevel 0 (
	set ISINSTALLED_JRE64=true
) else (
	set ISINSTALLED_JRE64=false
)


:: ******** Check input prameters *******
if /i [%1] NEQ [-b] if /i [%1] NEQ [-rc] if /i [%1] NEQ [-ra] goto PROMPT
if /i [%1] EQU [-b] (
	call:NOTED_BACKUPDB
	goto BACKUP
)
if /i [%1] EQU [-rc] (
	call:NOTED_SETUP
	call:NOTED_RESTOREDB
	goto RESTORE
)
if /i [%1] EQU [-ra] (
	call:NOTED_SETUP
	call:NOTED_DELETEBACKUP
	goto PREPARE_RESTORE
)


:BACKUP
echo === Step1. Backup certificate files from TMSM install folder ===
if not exist "%BACKUP_TMSM_FOLDER%" mkdir "%BACKUP_TMSM_FOLDER%"
if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%" mkdir "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%"
if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN32%" mkdir "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN32%"
if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN64%" mkdir "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN64%"
:: ******** Copy files from OSCE backup folder to correspond backup folder
::copy activemq.xml
call:FILECOMPARE "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"

:: copy broker.pem
call:FILECOMPARE "%TMSM_PATH%\%APACHE_CONF_PATH%\%BROKER_PEM_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_PEM_PATH%"
	
::copy broker.cert
call:FILECOMPARE "%TMSM_PATH%\%APACHE_CONF_PATH%\%BROKER_CERT_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_CERT_PATH%"

:: copy broker.ks
call:FILECOMPARE "%TMSM_PATH%\%APACHE_CONF_PATH%\%BROKER_KS_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_KS_PATH%"

:: copy wrapper.conf
call:FILECOMPARE "%TMSM_PATH%\%APACHE_BIN_WIN32%\%WRAPPER_CONF_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN32%\%WRAPPER_CONF_PATH%"
call:FILECOMPARE "%TMSM_PATH%\%APACHE_BIN_WIN64%\%WRAPPER_CONF_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN64%\%WRAPPER_CONF_PATH%"

::copy serverinfo.plist
call:FILECOMPARE "%TMSM_PATH%\%SERVERINFO_PLIST_PATH%" "%BACKUP_TMSM_FOLDER%\%SERVERINFO_PLIST_PATH%"

echo.
if "%COMPAREFILESSUCCESS%" EQU "false" (
	echo ******** Backup TMSM certificate files failed. ********
	goto END
) else (
	echo === Backup TMSM certificate files completed. ===
)
echo.


:: Backup TMSM database
if "%IsBackupDB%" EQU "true" (
	echo === Step2. Backup the SQL server data ===
	call:BACKUPDB "%cd%\%TMSMSQL_BAK%"
)


echo === Backup TMSM files finished. ===
goto END


:PREPARE_RESTORE
echo === Step1. Backup certificate files from backed up folder ===
:: ******** check TMSM BACKUP files exist (supported after TMSM 2.0) ********
if not exist "%TMSM_BACKUP_PATH%" (
	echo ******** TMSM Backup file does not exist. ********
	GOTO END
)
if not exist "%BACKUP_TMSM_FOLDER%" mkdir "%BACKUP_TMSM_FOLDER%"
if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%" mkdir "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%"
if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN32%" mkdir "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN32%"
if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN64%" mkdir "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN64%"
:: ******** Copy files from OSCE backup folder to correspond backup folder
::copy activemq.xml
call:FILECOMPARE "%TMSM_BACKUP_PATH%\%ACTIVEMQ_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"

:: copy broker.pem
call:FILECOMPARE "%TMSM_BACKUP_PATH%\%BROKER_PEM_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_PEM_PATH%"
	
::copy broker.cert
call:FILECOMPARE "%TMSM_BACKUP_PATH%\%BROKER_CERT_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_CERT_PATH%"

:: copy broker.ks
call:FILECOMPARE "%TMSM_BACKUP_PATH%\%BROKER_KS_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_KS_PATH%"

:: copy wrapper.conf
call:FILECOMPARE "%TMSM_BACKUP_PATH%\%WRAPPER32_CONF_PATH%\%WRAPPER_CONF_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN32%\%WRAPPER_CONF_PATH%"
call:FILECOMPARE "%TMSM_BACKUP_PATH%\%WRAPPER64_CONF_PATH%\%WRAPPER_CONF_PATH%" "%BACKUP_TMSM_FOLDER%\%APACHE_BIN_WIN64%\%WRAPPER_CONF_PATH%"

::copy serverinfo.plist
call:FILECOMPARE "%TMSM_BACKUP_PATH%\%SERVERINFO_PLIST_PATH%" "%BACKUP_TMSM_FOLDER%\%SERVERINFO_PLIST_PATH%"

echo.
if "%COMPAREFILESSUCCESS%" EQU "false" (
	echo ******** Backup TMSM certificate files failed. ********
	goto END
) else (
	echo === Backup TMSM certificate files completed. ===
)
echo.


echo === Backup TMSM certificate files completed. ===
goto RESTORE


:: ******** Restore TMSM certificate files ********
:RESTORE
:: ******** Check backup files exist ********
set IsExist=true
if not exist "%BACKUP_TMSM_FOLDER%" (set IsExist=false)
if "%IsExist%" EQU "false" (
	echo Backup files not existing, please backup TMSM certificate files first.
	goto END
)
:: ******** Check TMSM version ********
:: Check broker.pem exist or not to get TMSM version of the backup files
if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_PEM_PATH%" (set ISUPGRADEDFROM_15=true)
:: Check broker.pem and TMSM registry to get TMSM version
reg query "HKEY_LOCAL_MACHINE\SOFTWARE\%WOW6432NODE%TrendMicro\TMSM" /v "Version" >nul 2>&1
if %errorlevel% NEQ 0 if not exist "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%BROKER_PEM_PATH%" (set ISTMSM_15=true)
if exist "%TMSM_PATH%\%APACHE_BIN_PATH%\%APACHE_BIN_FILE%" (set ISTMSM_30=true)
:: ******** Generate _Copy.vbs ********
if exist %COPY_VBS% del %COPY_VBS%
echo Set fso = CreateObject("Scripting.FileSystemObject") >> %COPY_VBS%
echo Set objArgs = WScript.Arguments >>%COPY_VBS%
echo InputFolder = fso.GetAbsolutePathName(objArgs(0)) >> %COPY_VBS%
echo OutputFolder = fso.GetAbsolutePathName(objArgs(1)) >> %COPY_VBS%
echo Set objShell = CreateObject("Shell.Application") >> %COPY_VBS%
echo fso.CopyFolder InputFolder, OutputFolder >> %COPY_VBS%
echo WScript.Sleep 3000 >> %COPY_VBS%

:: ******** Generate _SubString.vbs ********
if exist %SUBSTRING_VBS% del %SUBSTRING_VBS%
echo Set objArgs = WScript.Arguments >> %SUBSTRING_VBS%
echo line = objArgs(0) >> %SUBSTRING_VBS%
echo start = InStrRev(line, ":") + 1 >> %SUBSTRING_VBS%
echo last = InStr(line, "?") >> %SUBSTRING_VBS%
echo result = Mid(line, start, last-start) >> %SUBSTRING_VBS%
echo Result = result ' for exaple >> %SUBSTRING_VBS%
echo Wscript.echo Result >> %SUBSTRING_VBS%
echo.

echo === Step 2. Copy TMSM server password ===
:: Get Server password
for /F "delims=" %%i in ('FINDSTR /L "username=\"server\"" "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"') do (set SrvPassword=%%i)
echo.


echo === Step 3. Overwrite certificate files ===
:: Copy the TMSM certificate files to be Overwritten
for /F "usebackq tokens=1,2 delims==" %%i in (`wmic os get LocalDateTime /VALUE 2^>NUL`) do if '.%%i.'=='.LocalDateTime.' set ldt=%%j
set folderDT=%ldt:~0,4%_%ldt:~4,2%_%ldt:~6,2%_%ldt:~8,2%_%ldt:~10,2%
if not exist %folderDT% mkdir %folderDT%
setlocal EnableDelayedExpansion
for /L %%n in (1 1 500) do if "!__cd__:~%%n,1!" neq "" set /a "len=%%n+1"
setlocal DisableDelayedExpansion
for /r "%BACKUP_TMSM_FOLDER%" %%f in (*) do (
  set "absPath=%%f"  
  setlocal EnableDelayedExpansion
  set "relPath=!absPath:~%len%!"
  if exist "%TMSM_PATH%\..\!relPath!" (echo f|xcopy "%TMSM_PATH%\..\!relPath!" "%folderDT%\!relPath!" > nul)
  endlocal
)
:: Overwrite certificate files
CScript //nologo %COPY_VBS% "%BACKUP_TMSM_FOLDER%" "%TMSM_PATH%"
echo.


echo === Step 4. Paste TMSM server password ===
:: Paste TMSM server password
if exist "%TEMP_ACTIVEMQ_PATH%" del "%TEMP_ACTIVEMQ_PATH%"
setlocal DisableDelayedExpansion
for /F "tokens=* delims=" %%i in ('type "%BACKUP_TMSM_FOLDER%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"') do (
	set line=%%i
	set lineRel=%%i
	setlocal EnableDelayedExpansion
	set lineRel=!lineRel:server" password=!
	if "!lineRel!" NEQ "!line!" (
		echo !SrvPassword!>>%TEMP_ACTIVEMQ_PATH%
	) else (
		echo/!line!>>%TEMP_ACTIVEMQ_PATH%
	)
	endlocal
)
del "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"
rename "%TEMP_ACTIVEMQ_PATH%" "%ACTIVEMQ_PATH%"
copy "%ACTIVEMQ_PATH%" "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"
del "%ACTIVEMQ_PATH%"
echo.


echo === Step 5. Update TMSM configuration files ===
:: Update wrapper_win32.conf, wrapper_win64.conf, and activemq.xml if TMSM server is 3.0
if exist "%TEMP_WRAPPER64_CONF_PATH%" del "%TEMP_WRAPPER64_CONF_PATH%"
if exist "%TEMP_WRAPPER32_CONF_PATH%" del "%TEMP_WRAPPER32_CONF_PATH%"
if exist "%TEMP_ACTIVEMQ_PATH%" del "%TEMP_ACTIVEMQ_PATH%"
setlocal DisableDelayedExpansion
if "%ISTMSM_30%" EQU "true" (
	for /F "tokens=* delims=" %%i in ('findstr /n "^" "%TMSM_PATH%\%APACHE_BIN_WIN64%\%WRAPPER_CONF_PATH%"') do (
		set line=%%i
		setlocal EnableDelayedExpansion
		set line=!line:*:=!
		if defined line (
			echo(!line:run.jar=%APACHE_BIN_FILE%!>>%TEMP_WRAPPER64_CONF_PATH%
		) else (
			echo(>>%TEMP_WRAPPER64_CONF_PATH%
		)
		endlocal
	)
	del "%TMSM_PATH%\%APACHE_BIN_WIN64%\%WRAPPER_CONF_PATH%"
	copy "%TEMP_WRAPPER64_CONF_PATH%" "%TMSM_PATH%\%APACHE_BIN_WIN64%\%WRAPPER_CONF_PATH%"
	del "%TEMP_WRAPPER64_CONF_PATH%"

	for /F "tokens=* delims=" %%i in ('findstr /n "^" "%TMSM_PATH%\%APACHE_BIN_WIN32%\%WRAPPER_CONF_PATH%"') do (
		set line=%%i
		setlocal EnableDelayedExpansion
		set line=!line:*:=!
		if defined line (
			echo(!line:run.jar=%APACHE_BIN_FILE%!>>%TEMP_WRAPPER32_CONF_PATH%
		) else (
			echo(>>%TEMP_WRAPPER32_CONF_PATH%
		)
		endlocal
	)
	del "%TMSM_PATH%\%APACHE_BIN_WIN32%\%WRAPPER_CONF_PATH%"
	copy "%TEMP_WRAPPER32_CONF_PATH%" "%TMSM_PATH%\%APACHE_BIN_WIN32%\%WRAPPER_CONF_PATH%"
	del "%TEMP_WRAPPER32_CONF_PATH%"

	for /F "tokens=* delims=" %%i in ('findstr /n "^" "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"') do (
		set line=%%i
		setlocal EnableDelayedExpansion
		set line=!line:*:=!
		if defined line (
			set lineRel=!line:ssl://0.0.0.0:=!
			if "!lineRel!" NEQ "!line!" (
				echo(!line:1048576^"/^>=1048576^&amp;transport.enabledProtocols=TLSv1,TLSv1.1,TLSv1.2^"/^>!>>%TEMP_ACTIVEMQ_PATH%
			) else (
				echo(!line!>>%TEMP_ACTIVEMQ_PATH%
			)
		) else (
			echo(>>%TEMP_ACTIVEMQ_PATH%
		)
		endlocal
	)
	del "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"
	copy "%TEMP_ACTIVEMQ_PATH%" "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"
	del "%TEMP_ACTIVEMQ_PATH%"
)
echo.


echo === Step 6. Overwrite certificate files to install package ===
:: Get Server Port
for /F "delims=" %%i in ('FINDSTR /L "name=\"ssl\"" "%TMSM_PATH%\%APACHE_CONF_PATH%\%ACTIVEMQ_PATH%"') do (set SrvPort=%%i)
for /f "delims=" %%a in ('CScript //nologo %SUBSTRING_VBS% "%SrvPort:"=""%"') do (set SrvPort=%%a)
:: Run serverinfo tool to update certificate files to install package. 
:: (After TMSM 2.0, run command to change firewall's port)
set tempHomeDir=%cd%
cd "%TMSM_PATH%"
if "%ISTMSM_15%" EQU "true" (
	"%SERVERINFO_PATH%" %PROTOCOL% %SrvPort% "%OSCEINI_PATH%" "%MQ_CONFIG_PATH%" "%TMSMINSTALL_MPKG_PATH%" "%PRODUCT_PATH%\%INATALL_MPKG_ZIP_PATH%" "%SERVERINI_PATH%"
	"%SERVERINFO_PATH%" --dir-acl "%CLIENTINSTALL_PATH%" S-1-5-32-546
) else (
	reg add "HKEY_LOCAL_MACHINE\SOFTWARE\%WOW6432NODE%TrendMicro\TMSM\CommunicationSetting" /f /v "ServerPort" /t REG_DWORD /d "%SrvPort%"
	"%SERVERINFO_PATH%" %PROTOCOL% "%OSCEINI_PATH%" "%MQ_CONFIG_PATH%" "%CLIENTINSTALL_PATH%" "%PRODUCT_PATH%" "%SERVERINI_PATH%" "%TMSM_HOME_DIR_PATH%" %ISINSTALLED_JRE64% %ISUPGRADEDFROM_15%
	"%SERVERINFO_PATH%" --dir-acl "%CLIENTINSTALL_PATH%" S-1-5-32-546
	call:ADDFIREWALL %SrvPort%
)
cd "%tempHomeDir%"
echo.


echo === Step 7. Restart TMSM services and restore SQL data ===
:: ******** Restart TMSM services ********
:: Stop services

call:STOPSERVICES OfcAosMgr
call:STOPSERVICES TMSMMainService
call:STOPSERVICES ActiveMQ4TMSM


:: Restore TMSM database
if "%IsRestoreDB%" EQU "true" (
	:: Copy TMSM database to be Overwritten
	call:BACKUPDB "%cd%\%folderDT%\%TMSMSQL_BAK%"
	call:RESTOREDB "%cd%\%TMSMSQL_BAK%"
)
:: Start services

call:STARTSERVICES ActiveMQ4TMSM
call:STARTSERVICES TMSMMainService
call:STARTSERVICES OfcAosMgr

echo.


:: ******** Delete files generated by system ********
if exist %COPY_VBS% del %COPY_VBS%
if exist %SUBSTRING_VBS% del %SUBSTRING_VBS%
if "%IsDeleteBackup%" EQU "true" (call:DELETEBACKUP)

echo === Recover the TMSM settings completed. ===
goto END


:FILECOMPARE
set TAR=%~1
set DES=%~2
if exist "%TAR%" (
	copy "%TAR%" "%DES%"	
	fc "%TAR%" "%DES%" >nul
	if errorlevel 1 (
		echo Compared error: Some difference between "%TAR% and "%DES%".
		set COMPAREFILESSUCCESS=false
	)
)
goto:EOF



:BACKUPDB
call:CHANGESQLACCOUNT "localsystem"
sqlCmd -S .\TMSM -E -Q "BACKUP DATABASE db_TMSM TO DISK='%~1';"
call:CHANGESQLACCOUNT "NT AUTHORITY\Network Service"
echo === Backup TMSM database to %~1 completed. ===
echo.
goto:EOF


:RESTOREDB
call:CHANGESQLACCOUNT "localsystem"
:: Restore SQL database
if exist "%~1" (
	sqlCmd -S .\TMSM -E -Q "RESTORE DATABASE db_TMSM FROM DISK='%~1' with replace;"
	if exist "%TMSMINSTALL_MPKG_PATH%" "%SERVERINFO_PATH%" --update-db-client-ver "%TMSMINSTALL_MPKG_PATH%"
	if exist "%TMSMINSTALL_PATH%" "%SERVERINFO_PATH%" --update-db-client-ver "%TMSMINSTALL_PATH%"
) else (
	echo Restore TMSM database failed. (Can't find "%~1")
)
call:CHANGESQLACCOUNT "NT AUTHORITY\Network Service"
goto:EOF


:DELETEBACKUP
if exist "%BACKUP_TMSM_FOLDER%" rmdir "%BACKUP_TMSM_FOLDER%" /s /q
goto:EOF


:NOTED_SETUP
echo.
echo ******** Note ********
echo 1.Please use PSTool (PsExec.exe -s -i cmd.exe) to open the cmd with system privilege then run this batch file.
echo 2.Please ensure you have login Windows as administrator session.
echo.
set /P ANSWER=Have you already setup the settings as noted above? [Y/N]?
if /i "%ANSWER%" EQU "N" (call:__ForceExit 2> nul)
if /i "%ANSWER%" EQU "NO" (call:__ForceExit 2> nul)
goto:EOF


:NOTED_BACKUPDB
echo.
echo ******** Note ********
set /P ANSWER=Do you want to backup TMSM database? [Y/N]?
if /i "%ANSWER%" EQU "Y" (set IsBackupDB=true)
if /i "%ANSWER%" EQU "YES" (set IsBackupDB=true)
goto:EOF


:NOTED_RESTOREDB
echo.
echo ******** Note ********
echo 1.Please do not restore database from TMSM 2.0 to TMSM 3.0 or after. 
set /P ANSWER=Do you want to recover TMSM database? [Y/N]?
if /i "%ANSWER%" EQU "Y" (set IsRestoreDB=true)
if /i "%ANSWER%" EQU "YES" (set IsRestoreDB=true)
goto:EOF


:NOTED_DELETEBACKUP
echo.
echo ******** Note ********
set /P ANSWER=Do you want to save backed up files after recovering? [Y/N]?
if /i "%ANSWER%" EQU "N" (set IsDeleteBackup=true)
if /i "%ANSWER%" EQU "NO" (set IsDeleteBackup=true)
goto:EOF


:CHANGESQLACCOUNT
sc config "%TMSMSQL_SERVICES%" obj= "%~1"
net stop "%TMSMSQL_SERVICES%"
net start "%TMSMSQL_SERVICES%"
timeout /t 3 >nul
goto:EOF


:ADDFIREWALL
if exist %TEMP_OSNAME_PATH% del %TEMP_OSNAME_PATH%
if not exist %SystemRoot%\system32\systeminfo.exe goto:EOF
systeminfo | find "OS Name" > %TEMP_OSNAME_PATH%
FOR /F "usebackq delims=: tokens=2" %%i IN (%TEMP_OSNAME_PATH%) DO set vers=%%i
echo %vers% | find "2003" > nul
if %ERRORLEVEL% EQU 0 (set OS2k8OrAfter=false)
if exist %TEMP_OSNAME_PATH% del %TEMP_OSNAME_PATH%
if "%OS2k8OrAfter%" EQU "true" (
	netsh advfirewall firewall add rule name= "ActiveMQ for Trend Micro Security" dir=in action=allow protocol=TCP localport=%~1
) else (
	netsh firewall add portopening protocol=TCP port=%~1 name= "ActiveMQ for Trend Micro Security"
)
goto:EOF


:STARTSERVICES
@echo off
setLocal EnableDelayedExpansion
net start "%~1"
for /F "tokens=3 delims=: " %%H in ('sc query "%~1" ^| findstr "        STATE"') do (
	set /a Counter+=1
	if !Counter! gtr 3 (
		echo Can't start "%~1", please manual to start it.
		echo.
		goto:EOF
	)	
	if /I "%%H" NEQ "RUNNING" (		
		echo Start "%~1" failed, retry again.
		timeout /t 3 >nul
		goto:STARTSERVICES
	) else (
		goto:EOF
	)
)
goto:EOF


:STOPSERVICES
@echo off
setLocal EnableDelayedExpansion
net stop "%~1" /y
for /F "tokens=3 delims=: " %%H in ('sc query "%~1" ^| findstr "        STATE"') do (
	set /a Counter+=1
	if !Counter! gtr 3 (
		echo Can't stop "%~1", please manual to stop it.
		echo.
		goto:EOF
	)	
	if /I "%%H" NEQ "STOPPED" (		
		echo Stop "%~1" failed, retry again.
		timeout /t 3 >nul
		goto:STOPSERVICES
	) else (
		goto:EOF
	)
)
goto:EOF


:PROMPT
echo Please specify the prameters
echo -b : Backup TMSM files
echo -rc: Restore TMSM files
echo -ra: Backup and restore TMSM files (Supported after TMSM 2.0)
goto END


:__ForceExit
:: Creates a syntax error, stops immediately
()
goto:EOF


:END
goto:EOF